import { AbstractControl } from '@angular/forms';
import { Observable } from 'rxjs';
import { NgFormsManagerConfig } from './config';
import { Control, FormKeys, UpsertConfig } from './types';
import * as i0 from "@angular/core";
export declare class NgFormsManager<FormsState = any> {
    private config;
    private readonly browserStorage?;
    private readonly store;
    private valueChanges$$;
    private instances$$;
    private initialValues$$;
    private destroy$$;
    constructor(config: NgFormsManagerConfig, browserStorage?: Storage);
    /**
     *
     * @example
     *
     * Whether the control is valid
     *
     * const valid$ = manager.validityChanges('login');
     *
     */
    validityChanges(name: keyof FormsState, path?: string): Observable<boolean>;
    /**
     *
     * Whether the control is valid
     *
     * @example
     *
     * manager.isValid(name);
     *
     */
    isValid(name: keyof FormsState): boolean;
    /**
     *
     * @example
     *
     * Whether the control is dirty
     *
     * const dirty$ = manager.dirtyChanges('login');
     *
     */
    dirtyChanges(name: keyof FormsState, path?: string): Observable<boolean>;
    /**
     *
     * @example
     *
     * Whether the control is disabled
     *
     * const disabled$ = manager.disableChanges('login');
     *
     */
    disableChanges(name: keyof FormsState, path?: string): Observable<boolean>;
    /**
     *
     * @example
     *
     * Observe the control's value
     *
     * const value$ = manager.valueChanges('login');
     * const value$ = manager.valueChanges<string>('login', 'email');
     *
     */
    valueChanges<T = any>(name: keyof FormsState, path: string): Observable<T>;
    valueChanges<T extends keyof FormsState>(name: T, path?: string): Observable<FormsState[T]>;
    /**
     *
     * @example
     *
     * Observe the control's errors
     *
     * const errors$ = manager.errorsChanges<Errors>('login');
     * const errors$ = manager.errorsChanges<Errors>('login', 'email');
     *
     */
    errorsChanges<Errors = any>(name: keyof FormsState, path?: string): Observable<Errors>;
    /**
     *
     * @example
     *
     * Observe the control's state
     *
     * const control$ = manager.controlChanges('login');
     * const control$ = manager.controlChanges<string>('login', 'email');
     *
     */
    controlChanges<State = any>(name: keyof FormsState, path: string): Observable<Control<State>>;
    controlChanges<T extends keyof FormsState>(name: T, path?: string): Observable<Control<FormsState[T]>>;
    /**
     *
     * Whether the initial control value is deep equal to current value
     *
     * @example
     *
     * const dirty$ = manager.initialValueChanged('settings');
     *
     */
    initialValueChanged(name: keyof FormsState): Observable<boolean>;
    /**
     *
     * @example
     *
     * Emits when the control is destroyed
     *
     * const control$ = manager.controlChanges('login').pipe(takeUntil(controlDestroyed('login')))
     *
     */
    controlDestroyed(name: keyof FormsState): Observable<unknown>;
    /**
     *
     * @example
     *
     * Get the control's state
     *
     * const control = manager.getControl('login');
     * const control = manager.getControl<string>('login', 'email');
     *
     */
    getControl<State = any>(name: keyof FormsState, path: string): Control<State> | null;
    getControl<T extends keyof FormsState>(name: T, path?: string): Control<FormsState[T]> | null;
    /**
     *
     * Get the initial value for a control
     *
     * Will return undefined, if no initial value was returned.
     *
     * @example
     *
     * manager.getInitialValue('login');
     *
     */
    getInitialValue<State = any>(name: keyof FormsState): State | undefined;
    getInitialValue<T extends keyof FormsState>(name: keyof FormsState): FormsState[T] | undefined;
    /**
     *
     * @example
     *
     *  Whether the form exists
     *
     * manager.hasControl('login');
     * manager.hasControl('login', 'email');
     *
     */
    hasControl(name: keyof FormsState, path?: string): boolean;
    /**
     *
     * @example
     *
     * A proxy to the original `patchValue` method
     *
     * manager.patchValue('login', { email: '' });
     *
     */
    patchValue<T extends keyof FormsState>(name: T, value: Partial<FormsState[T]>, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `setValue` method
     *
     * manager.setValue('login', { email: '', name: '' });
     *
     */
    setValue<T extends keyof FormsState>(name: keyof FormsState, value: FormsState[T], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `reset` method
     *
     * manager.reset('login', { email: '' });
     *
     */
    reset<T extends keyof FormsState>(name: T, value?: Partial<FormsState[T]>, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * Sets the initial value for a control
     *
     * @example
     *
     * manager.setInitialValue('login', value);
     *
     */
    setInitialValue(name: keyof FormsState, value: any): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAllAsTouched` method
     *
     * manager.markAllAsTouched('login');
     *
     */
    markAllAsTouched(name: keyof FormsState): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAsTouched` method
     *
     * manager.markAsTouched('login');
     *
     */
    markAsTouched(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * Marks the control and all its descendant controls as dirty.
     *
     * manager.markAllAsDirty('login');
     *
     */
    markAllAsDirty(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAsDirty` method
     *
     * manager.markAsDirty('login');
     *
     */
    markAsDirty(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAsPending` method
     *
     * manager.markAsPending('login');
     *
     */
    markAsPending(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAsPristine` method
     *
     * manager.markAsPristine('login');
     *
     */
    markAsPristine(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * A proxy to the original `markAsUntouched` method
     *
     * manager.markAsUntouched('login');
     *
     */
    markAsUntouched(name: keyof FormsState, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     *
     * @example
     *
     * manager.unsubscribe('login');
     *
     */
    unsubscribe(name?: FormKeys<FormsState>): void;
    /**
     *
     * @example
     *
     * Removes the control from the store and from browser storage
     *
     * manager.clear('login');
     *
     */
    clear(name?: FormKeys<FormsState>): void;
    /**
     *
     * @example
     *
     * Calls unsubscribe and clear
     *
     * manager.destroy('login');
     *
     */
    destroy(name?: FormKeys<FormsState>): void;
    /**
     *
     * @example
     *
     * Register a control
     *
     * manager.upsert('login', this.login);
     * manager.upsert('login', this.login, { persistState: true });
     * manager.upsert('login', this.login, { debounceTime: 500 });
     *
     * manager.upsert('login', this.login, { arrControlFactory: value => new FormControl('') });
     *
     */
    upsert(name: keyof FormsState, control: AbstractControl, config?: UpsertConfig): this;
    private getValueChangeStreams;
    private removeFromStorage;
    private updateStorage;
    private getFromStorage;
    private deleteControl;
    private toControlValue;
    private updateStore;
    private removeInitialValue;
    private markDescendantsAsDirty;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgFormsManager<any>, [{ optional: true; }, null]>;
    static ɵprov: i0.ɵɵInjectableDeclaration<NgFormsManager<any>>;
}
